--[[ ArenaLive Core Functions: AbsorbBar Handler
Created by: Vadrak
Creation Date: 02.06.2013
Last Update: 06.06.2013
This file contains all relevant functions for health bars and their behaviour.
]]--
local addonName = ...;

-- Local version is said to be faster.
local ArenaLiveCore = ArenaLiveCore;

-- Set up a new handler.
local FrameMover = ArenaLiveCore:AddHandler("FrameMover", "EventCore");

-- Get the global UnitFrame, ArenaHeader and GroupHeader handlers.
local UnitFrame = ArenaLiveCore:GetHandler("UnitFrame");
local ArenaHeader = ArenaLiveCore:GetHandler("ArenaHeader");
local PartyHeader = ArenaLiveCore:GetHandler("PartyHeader");

-- Register all needed events
FrameMover:RegisterEvent("PLAYER_REGEN_ENABLED");

-- Set up a table to store all absorb bars. This way it is easier to update them.
local frameMoverToUpdate = {};

-- *** FRAME FUNCTIONS ***
--[[ Function: Enable
	 Enables the frame mover.
]]--
local function Enable (self, elapsed)
	if ( not InCombatLockdown() ) then
		frameMoverToUpdate[self] = nil;
		self.enabled = true;
		self.unitFrame:SetAttribute("alframelock", nil);
		self:SetScript("OnUpdate", self.OnUpdate);
		self:Show();
	else
		frameMoverToUpdate[self] = "enable";
	end
end

--[[ Function: Disable
	 Disables the frame mover.
]]--
local function Disable(self)
	if ( not InCombatLockdown() ) then
		frameMoverToUpdate[self] = nil;
		self.enabled = nil;
		self.unitFrame:SetAttribute("alframelock", true);
		self:SetScript("OnUpdate", nil);
		self:Hide();
	else
		frameMoverToUpdate[self] = "disable";
	end
end

local function OnUpdate (self, elapsed)

	if ( self.enabled ) then
		self:Update();
	end

end

local function OnShow (self)
	
	frameMover:ClearAllPoints();
	frameMover:SetAllPoints(frameMover.unitFrame);

end

local function OnDragStart (self, button)
	self.unitFrame:StartMoving();
	self.unitFrame:SetClampedToScreen(true);
end

local function OnDragStop (self)
	self.unitFrame:StopMovingOrSizing();
end

local function Update (self)

	if ( not self.enabled ) then
		return;
	end
	
	local frameName = self.unitFrame:GetName();
	local left, bottom, width, height = self:GetRect();
	local text = string.format(ArenaLiveCore:GetLocalisation(addonName, "FRAME_MOVER_TEXT"), frameName, left, bottom)
	
	self.text:SetText(text);

end

local function Reset (self)
	self:Disable();
	self.text:SetText();
end

-- *** HANDLER FUNCTIONS ***
function FrameMover:AddFrame (frameMover, fontString, unitFrame)

	-- Create a reference for the absorb bar inside the unit frame and vice versa.
	unitFrame.frameMover = frameMover;
	frameMover.unitFrame = unitFrame;
	
	-- Register in Handlerlist
	unitFrame.handlerList.frameMover = true;
	
	-- Register the fontstring that shows the coordinates of the farme.
	frameMover.text = fontString;
	
	frameMover.Enable = Enable;
	frameMover.Disable = Disable;
	frameMover.OnDragStart = OnDragStart;
	frameMover.OnDragStop = OnDragStop;
	frameMover.OnUpdate = OnUpdate;
	frameMover.Update = Update;
	frameMover.Reset = Reset;
	
	-- Set the size of the mover to be the same as the frame, if the frame is larget than the mover.
	frameMover:SetAllPoints(unitFrame);
	
	-- Register for drag
	frameMover:RegisterForDrag("LeftButton");
	
	-- Set Scripts
	frameMover:SetScript("OnDragStart", frameMover.OnDragStart);
	frameMover:SetScript("OnDragStop", frameMover.OnDragStop);
	frameMover:SetScript("OnShow", frameMover.OnShow);
	-- Enable or disable the mover according to saved variables
	local DBKey = "FrameLock"
	local isLocked = ArenaLiveCore:GetDBEntry(addonName, DBKey);	

	if ( isLocked ) then
		frameMover:Disable();
	else
		frameMover:Enable();
	end

	
end

function FrameMover:UpdateAll()

	-- Enable or disable all moverrs according to saved variables
	local DBKey = "FrameLock"
	local isLocked = ArenaLiveCore:GetDBEntry(addonName, DBKey);	
	
	for key, frame in ipairs(UnitFrame.UnitFrameTable) do
		if ( frame.handlerList.frameMover ) then
			if ( isLocked ) then
				frame.frameMover:Disable();
			else
				frame.frameMover:Enable();
			end	
		end
	end

	for header, value in pairs(ArenaHeader.HeaderTable) do
		if ( header.handlerList.frameMover ) then
			if ( isLocked ) then
				header.frameMover:Disable();
			else
				header.frameMover:Enable();
			end	
		end
	end	
	
	for header, value in pairs(PartyHeader.HeaderTable) do
		if ( header.handlerList.frameMover ) then
			if ( isLocked ) then
				header.frameMover:Disable();
			else
				header.frameMover:Enable();
			end	
		end
	end
	
end

function FrameMover:OnEvent(event, ...)

	if ( event == "PLAYER_REGEN_ENABLED" ) then
	
		for frame, update in pairs(frameMoverToUpdate) do
			if ( update == "enable" ) then
				frame:Enable();
			elseif ( update == "disable" ) then
				frame:Disable();
			end
		end
	
	end

end